/*
 * libmm-player
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: JongHyuk Choi <jhchoi.choi@samsung.com>, YeJin Cho <cho.yejin@samsung.com>,
 * Seungbae Shin <seungbae.shin@samsung.com>, YoungHwan An <younghwan_.an@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __MM_PLAYER_H__
#define	__MM_PLAYER_H__


/*===========================================================================================
|                                                                                           |
|  INCLUDE FILES                                        |
|                                                                                           |
========================================================================================== */

#include <glib.h>

#include <mm_types.h>
#include <mm_message.h>
#include <media_packet.h>

#ifdef __cplusplus
	extern "C" {
#endif

/*===========================================================================================
|                                                                                           |
|  GLOBAL DEFINITIONS AND DECLARATIONS                                        |
|                                                                                           |
========================================================================================== */

/**
 * MM_PLAYER_URI:
 *
 * uri to play (string)
 *
 */
#define MM_PLAYER_CONTENT_URI               "profile_uri"
/**
 * MM_PLAYER_VIDEO_ROTATION
 *
 * can change video angle (int)
 * @see MMDisplayRotationType
 */
#define MM_PLAYER_VIDEO_ROTATION            "display_rotation"
/**
 * MM_PLAYER_VIDEO_WIDTH:
 *
 * get the video width (int), It's guaranteed after calling mm_player_start() or
 * receiving MM_MESSAGE_BEGIN_OF_STREAM.
 *
 */
#define MM_PLAYER_VIDEO_WIDTH               "content_video_width"
/**
 * MM_PLAYER_VIDEO_HEIGHT:
 *
 * get the video height (int), It's guaranteed after calling mm_player_start() or
 * receiving MM_MESSAGE_BEGIN_OF_STREAM.
 *
 */
#define MM_PLAYER_VIDEO_HEIGHT              "content_video_height"

/**
 * MM_PLAYER_VIDEO_FPS:
 */
#define MM_PLAYER_VIDEO_FPS                 "content_video_fps"

/**
 * MM_PLAYER_VIDEO_BITRATE:
 */
#define MM_PLAYER_VIDEO_BITRATE             "content_video_bitrate"

/**
 * MM_PLAYER_VIDEO_EVAS_SURFACE_SINK:
 *
 * get the video evas surface sink plugin name (string), It's guaranteed after calling mm_player_create()
 *
 */
#define MM_PLAYER_VIDEO_EVAS_SURFACE_SINK          "display_evas_surface_sink"
/**
 * MM_PLAYER_MEM_SRC:
 *
 * set memory pointer to play (data)
 *
 */
#define MM_PLAYER_MEMORY_SRC                "profile_user_param"
/**
 * MM_PLAYER_PLAYBACK_COUNT
 *
 * can set playback count (int), Default value is 1 and -1 is for infinity playing until releasing it.
 *
 */
#define MM_PLAYER_PLAYBACK_COUNT            "profile_play_count"
/**
 * MM_PLAYER_SUBTITLE_URI
 *
 * set the subtitle path (string)
 */
#define MM_PLAYER_SUBTITLE_URI              "subtitle_uri"
/**
 * MM_PLAYER_STREAMING_USER_AGENT
 *
 * set the streaming user agent (string)
 */
#define MM_PLAYER_STREAMING_USER_AGENT      "streaming_user_agent"
/**
 * MM_PLAYER_STREAMING_COOKIE
 *
 * set the streaming cookie (int)
 */
#define MM_PLAYER_STREAMING_COOKIE          "streaming_cookie"
/**
 * MM_PLAYER_VIDEO_CODEC
 *
 * codec the video data is stored in (string)
 */
#define MM_PLAYER_VIDEO_CODEC               "content_video_codec"
/**
 * MM_PLAYER_AUDIO_CODEC
 *
 * codec the audio data is stored in (string)
 */
#define MM_PLAYER_AUDIO_CODEC               "content_audio_codec"
/**
 * MM_PLAYER_AUDIO_BITRATE
 *
 * set the streaming proxy port (int)
 */
#define MM_PLAYER_AUDIO_BITRATE             "content_audio_bitrate"
/**
 * MM_PLAYER_AUDIO_CHANNEL
 *
 * the number of audio channel (int)
 */
#define MM_PLAYER_AUDIO_CHANNEL             "content_audio_channels"
/**
 * MM_PLAYER_AUDIO_SAMPLERATE
 *
 * audio samplerate  (int)
 */
#define MM_PLAYER_AUDIO_SAMPLERATE          "content_audio_samplerate"
/**
 * MM_PLAYER_TEXT_TRACK_NUM
 *
 * track number inside a collection (int)
 */
#define MM_PLAYER_TEXT_TRACK_NUM            "content_text_track_num"
/**
 * MM_PLAYER_TAG_ARTIST
 *
 * person(s) responsible for the recording (string)
 */
#define MM_PLAYER_TAG_ARTIST                "tag_artist"
/**
 * MM_PLAYER_TAG_TITLE
 *
 * title (string)
 */
#define MM_PLAYER_TAG_TITLE                 "tag_title"
/**
 * MM_PLAYER_TAG_ALBUM
 *
 * album containing this data (string)
 */
#define MM_PLAYER_TAG_ALBUM                 "tag_album"
/**
 * MM_PLAYER_TAG_GENRE
 *
 * genre this data belongs to (string)
 */
#define MM_PLAYER_TAG_GENRE                 "tag_genre"
/**
 * MM_PLAYER_TAG_AUTHOUR
 *
 * author (string)
 */
#define MM_PLAYER_TAG_AUTHOUR               "tag_author"
/**
 * MM_PLAYER_TAG_COPYRIGHT
 *
 * copyright notice of the data (string)
 */
#define MM_PLAYER_TAG_COPYRIGHT             "tag_copyright"
/**
 * MM_PLAYER_TAG_DATE
 *
 * date the data was created (string)
 */
#define MM_PLAYER_TAG_DATE                  "tag_date"
/**
 * MM_PLAYER_TAG_DESCRIPRION
 *
 * short text describing the content of the data (string)
 */
#define MM_PLAYER_TAG_DESCRIPRION           "tag_description"
/**
 * MM_PLAYER_TAG_TRACK_NUM
 *
 * track number inside a collection (int)
 */
#define MM_PLAYER_TAG_TRACK_NUM             "tag_track_num"

/**
 * MM_PLAYER_DRC_MODE
 *
 * dynamic resolution change mode (int)
 */
#define MM_PLAYER_DRC_MODE                  "drc_mode"

/**
 * MM_PLAYER_GAPLESS_MODE
 *
 * gapless playback mode (int)
 */
#define MM_PLAYER_GAPLESS_MODE              "gapless_mode"

/**
 * MM_PLAYER_ENABLE_VIDEO_DECODED_CB
 *
 * enable video decoded callback (int)
 */
#define MM_PLAYER_ENABLE_VIDEO_DECODED_CB   "enable_video_decoded_cb"

/**
 * MM_PLAYER_VIDEO_CODEC_TYPE
 *
 * video codec type (int)
 */
#define MM_PLAYER_VIDEO_CODEC_TYPE          "video_codec_type"

/**
 * MM_PLAYER_AUDIO_CODEC_TYPE
 *
 * audio codec type (int)
 */
#define MM_PLAYER_AUDIO_CODEC_TYPE          "audio_codec_type"

/**
 * MM_PLAYER_VIDEO_BUFFER_TOTAL_SIZE (int)
 */
#define MM_PLAYER_VIDEO_BUFFER_TOTAL_SIZE   "video_buffer_total_size"

/**
 * MM_PLAYER_VIDEO_BUFFER_EXTRA_SIZE (int)
 */
#define MM_PLAYER_VIDEO_BUFFER_EXTRA_SIZE   "video_buffer_extra_size"

/**
 * MM_PLAYER_PREBUFFER_MS
 *
 * prebuffer ms (int)
 */
#define MM_PLAYER_PREBUFFER_MS              "prebuffer_ms"

/**
 * MM_PLAYER_REBUFFER_MS
 *
 * rebuffer ms (int)
 */
#define MM_PLAYER_REBUFFER_MS               "rebuffer_ms"

/**
 * MM_PLAYER_AUDIO_OFFLOAD (int)
 */
#define MM_PLAYER_AUDIO_OFFLOAD             "audio_offload"

/**
 * MM_PLAYER_AUDIO_OFFLOAD_FORMAT (int array)
 */
#define MM_PLAYER_AUDIO_OFFLOAD_FORMAT      "audio_offload_format"

/**
 * MM_PLAYER_PITCH_CONTROL (int)
 */
#define MM_PLAYER_PITCH_CONTROL             "pitch_control"

/**
 * MM_PLAYER_PITCH_VALUE (double)
 */
#define MM_PLAYER_PITCH_VALUE               "pitch_value"

/**
 * MM_PLAYER_PCM_EXT_FORMAT (string)
 */
#define MM_PLAYER_PCM_EXT_FORMAT            "pcm_extraction_format"

/**
 * MM_PLAYER_PCM_EXT_CHANNELS (int)
 */
#define MM_PLAYER_PCM_EXT_CHANNELS          "pcm_extraction_channels"

/**
 * MM_PLAYER_PCM_EXT_SAMPLERATE (int)
 */
#define MM_PLAYER_PCM_EXT_SAMPLERATE        "pcm_extraction_samplerate"

/**
 * MM_PLAYER_MEDIA_STREAM_INPUT_FORMAT (int array)
 */
#define MM_PLAYER_MEDIA_STREAM_INPUT_FORMAT "media_stream_input_format"

#define BUFFER_MAX_PLANE_NUM (4)

/**
 * Enumerations of player state.
 */
typedef enum {
	MM_PLAYER_STATE_NULL,				/**< Player is created, but not realized yet */
	MM_PLAYER_STATE_READY,				/**< Player is ready to play media */
	MM_PLAYER_STATE_PLAYING,			/**< Player is now playing media */
	MM_PLAYER_STATE_PAUSED,				/**< Player is paused while playing media */
	MM_PLAYER_STATE_NONE,				/**< Player is not created yet */
	MM_PLAYER_STATE_NUM,				/**< Number of player states */
} mmplayer_state_e;

/**
 * Enumeration for attribute values types.
 */
typedef enum {
	MM_PLAYER_ATTRS_TYPE_INVALID = -1,        /**< Type is invalid */
	MM_PLAYER_ATTRS_TYPE_INT,                 /**< Integer type */
	MM_PLAYER_ATTRS_TYPE_DOUBLE,              /**< Double type */
	MM_PLAYER_ATTRS_TYPE_STRING,              /**< UTF-8 String type */
	MM_PLAYER_ATTRS_TYPE_DATA,                /**< Pointer type */
	MM_PLAYER_ATTRS_TYPE_ARRAY,               /**< Array type */
	MM_PLAYER_ATTRS_TYPE_RANGE,               /**< Range type */
	MM_PLAYER_ATTRS_TYPE_NUM,                 /**< Number of attribute type */
} mmplayer_attrs_type_e;

/**
 * Enumeration for attribute validation type.
 */
typedef enum {
	MM_PLAYER_ATTRS_VALID_TYPE_INVALID = -1,		/**< Invalid validation type */
	MM_PLAYER_ATTRS_VALID_TYPE_NONE,				/**< Do not check validity */
	MM_PLAYER_ATTRS_VALID_TYPE_INT_ARRAY,          /**< validity checking type of integer array */
	MM_PLAYER_ATTRS_VALID_TYPE_INT_RANGE,          /**< validity checking type of integer range */
	MM_PLAYER_ATTRS_VALID_TYPE_DOUBLE_ARRAY,		/**< validity checking type of double array */
	MM_PLAYER_ATTRS_VALID_TYPE_DOUBLE_RANGE,       /**< validity checking type of double range */
} mmplayer_attrs_valid_type_e;

/**
 * Enumeration for attribute access flag.
 */
typedef enum {
	MM_PLAYER_ATTRS_FLAG_NONE = 0,					/**< None flag is set */
	MM_PLAYER_ATTRS_FLAG_READABLE = 1 << 0,			/**< Readable */
	MM_PLAYER_ATTRS_FLAG_WRITABLE = 1 << 1,			/**< Writable */
	MM_PLAYER_ATTRS_FLAG_MODIFIED = 1 << 2,			/**< Modified */

	MM_PLAYER_ATTRS_FLAG_RW = MM_PLAYER_ATTRS_FLAG_READABLE | MM_PLAYER_ATTRS_FLAG_WRITABLE, /**< Readable and Writable */
} mmplayer_attrs_flag_e;

/**
 * Enumeration of track types
 */
typedef enum {
	MM_PLAYER_TRACK_TYPE_AUDIO = 0,
	MM_PLAYER_TRACK_TYPE_VIDEO,
	MM_PLAYER_TRACK_TYPE_TEXT,
	MM_PLAYER_TRACK_TYPE_MAX
} mmplayer_track_type_e;

/**
 * Enumeration of runtime buffering mode
 */
typedef enum {
	MM_PLAYER_BUFFERING_MODE_ADAPTIVE = 0,	/**< default, If buffering is occurred, player will consider the bandwidth to adjust buffer setting. */
	MM_PLAYER_BUFFERING_MODE_FIXED,			/**< player will set buffer size with this fixed size value. */
	MM_PLAYER_BUFFERING_MODE_MAX,
} mmplayer_buffering_mode_e;

/**
 * Enumeration of media stream buffer status
 */
typedef enum {
	MM_PLAYER_MEDIA_STREAM_BUFFER_UNDERRUN,
	MM_PLAYER_MEDIA_STREAM_BUFFER_OVERFLOW,
} mmplayer_media_stream_buffer_status_e;

/**
 * Enumeration for stream type.
 */
typedef enum {
	MM_PLAYER_STREAM_TYPE_DEFAULT,	/**< Container type */
	MM_PLAYER_STREAM_TYPE_AUDIO,	/**< Audio element stream type */
	MM_PLAYER_STREAM_TYPE_VIDEO,	/**< Video element stream type */
	MM_PLAYER_STREAM_TYPE_TEXT,     /**< Text type */
	MM_PLAYER_STREAM_TYPE_MAX,
} mmplayer_stream_type_e;

typedef enum {
	MM_PLAYER_CODEC_TYPE_HW,          /**< HW codec can only be selected */
	MM_PLAYER_CODEC_TYPE_SW,          /**< SW codec can only be selected */
	MM_PLAYER_CODEC_TYPE_DEFAULT,     /**< codec is selected by the priority */
} mmplayer_codec_type_e;

typedef enum {
	MM_PLAYER_AUDIO_CONTROL_OPT_EFFECT,
	MM_PLAYER_AUDIO_CONTROL_OPT_REPLAYGAIN,
	MM_PLAYER_AUDIO_CONTROL_OPT_PITCH,
	MM_PLAYER_AUDIO_CONTROL_OPT_PCM_EXPORTING,
} mmplayer_audio_control_opt_e;

typedef enum {
	MM_PLAYER_AUDIO_EXTRACT_DEFAULT                        = 0x00,   /**< Sync with the playback clock and multichannel audio stream */
	MM_PLAYER_AUDIO_EXTRACT_NO_SYNC_WITH_CLOCK             = 0x01,   /**< No sync with the playback clock */
	MM_PLAYER_AUDIO_EXTRACT_DEINTERLEAVE                   = 0x02,   /**< Splits one interleaved multichannel audio stream into many mono audio streams */
	MM_PLAYER_AUDIO_EXTRACT_NO_SYNC_AND_DEINTERLEAVE       = MM_PLAYER_AUDIO_EXTRACT_NO_SYNC_WITH_CLOCK | MM_PLAYER_AUDIO_EXTRACT_DEINTERLEAVE,
	MM_PLAYER_AUDIO_EXTRACT_WITH_PLAYBACK                  = 0x04,   /**< With audio playback synchronously (experimental) */
	MM_PLAYER_AUDIO_EXTRACT_DEINTERLEAVE_WITH_PLAYBACK     = MM_PLAYER_AUDIO_EXTRACT_DEINTERLEAVE | MM_PLAYER_AUDIO_EXTRACT_WITH_PLAYBACK,
} mmplayer_audio_extract_opt_e;

/*
 * Enumerations of video colorspace
 */
typedef enum {
	MM_PLAYER_COLORSPACE_I420 = 0,			/**< I420 format - planer */
	MM_PLAYER_COLORSPACE_RGB888,			/**< RGB888 pixel format */
	MM_PLAYER_COLORSPACE_NV12_TILED,		/**< Customized color format */
	MM_PLAYER_COLORSPACE_NV12,
	MM_PLAYER_COLORSPACE_BGRx,				/**< BGRx pixel format */
	MM_PLAYER_COLORSPACE_MAX = 0x7FFFFFFF
} mmplayer_video_color_space_e;

/**
 * Attribute validity structure
 */
typedef struct {
	 mmplayer_attrs_type_e type;
	 mmplayer_attrs_valid_type_e validity_type;
	 mmplayer_attrs_flag_e flag;
	/**
	  * a union that describes validity of the attribute.
	  * Only when type is 'MM_ATTRS_TYPE_INT' or 'MM_ATTRS_TYPE_DOUBLE',
	  * the attribute can have validity.
	 */
	 union {
		/**
		   * Validity structure for integer array.
		 */
		struct {
			int *array;  /**< a pointer of array */
			int count;   /**< size of array */
			int d_val;
		} int_array;
		/**
		   * Validity structure for integer range.
		 */
		struct {
			int min;   /**< minimum range */
			int max;   /**< maximum range */
			int d_val;
		} int_range;
		/**
		* Validity structure for double array.
		*/
		struct {
			double *array;  /**< a pointer of array */
			int count;   /**< size of array */
			double d_val;
		} double_array;
		/**
		* Validity structure for double range.
		*/
		struct {
			double min;   /**< minimum range */
			double max;   /**< maximum range */
			double d_val;
		} double_range;
	};
} mmplayer_attrs_info_t;

typedef struct {
	MMPixelFormatType format;              /**< image format */
	int width;                             /**< width of video buffer */
	int height;                            /**< height of video buffer */
	unsigned int timestamp;                /**< timestamp of stream buffer (msec)*/
	unsigned int length_total;             /**< total length of stream buffer (in byte)*/
	void *data[BUFFER_MAX_PLANE_NUM];
	void *bo[BUFFER_MAX_PLANE_NUM];        /**< TBM buffer object */
	void *internal_buffer;                 /**< Internal buffer pointer */
	int stride[BUFFER_MAX_PLANE_NUM];      /**< stride of plane */
	int elevation[BUFFER_MAX_PLANE_NUM];   /**< elevation of plane */
	int orientation;                       /**< orientation */
	int bo_size;                           /**< TBM buffer object size */
	int plane_num;                         /**< number of Plane */
} mmplayer_video_decoded_data_info_t;

typedef struct {
	void *data;
	int data_size;
	int channel;
	int bitrate;
	guint64 channel_mask;
	media_format_mimetype_e pcm_format;
} mmplayer_audio_decoded_data_info_t;

typedef struct {
	unsigned char *data;					/* capture image buffer */
	int size;								/* capture image size */
	mmplayer_video_color_space_e fmt;			/* color space type */
	unsigned int width;						/* width of captured image */
	unsigned int height;					/* height of captured image */
	unsigned int orientation;				/* content orientation */
} mmplayer_video_capture_t;

/**
 * Video stream info in external demux case
 *
**/
typedef struct {
	const char *mime;
	unsigned int framerate_num;
	unsigned int framerate_den;
	unsigned int width;
	unsigned int height;
	unsigned char *codec_extradata;
	unsigned int extradata_size;
	unsigned int version;
} mmplayer_video_stream_info_t;

/**
 * Audio stream info in external demux case
 *
**/
typedef struct {
	const char *mime;
	unsigned int channels;
	unsigned int sample_rate;
	unsigned char *codec_extradata;
	unsigned int extradata_size;
	unsigned int version;
	unsigned int user_info;

	/* for pcm */
//	unsigned int width;
//	unsigned int depth;
//	unsigned int endianness;
//	bool signedness;
} mmplayer_audio_stream_info_t;

/**
 * Subtitle stream info in external demux case
 *
**/
typedef struct _SubtitleStreamInfo {
	const char *mime;
	unsigned int codec_tag;
	void *context;  //for smpte text
} mmplayer_subtitle_stream_info_t;

/**
 * Called to notify the stream changed.
 */
typedef bool (*mm_player_stream_changed_callback)(void *user_param);

/**
 * Video decoded callback function type.
 */
typedef bool (*mm_player_video_decoded_callback)(void *stream, void *user_param);

/**
 * Audio stream callback function type.
 */
typedef bool (*mm_player_video_capture_callback)(void *stream, int stream_size, void *user_param);

/**
 * Audio stream callback function type.
 */
typedef bool (*mm_player_audio_decoded_callback)(void *stream, void *user_param);

/*===========================================================================================
|                                                                                           |
|  GLOBAL FUNCTION PROTOTYPES                                        |
|                                                                                           |
========================================================================================== */

/**
 * This function creates a player object for playing multimedia contents. \n
 * The attributes of player are created to get/set some values with application. \n
 * And, mutex, gstreamer and other resources are initialized at this time. \n
 * If player is created, the state will become MM_PLAYER_STATE_NULL.
 *
 * @param   player [out] Handle of player
 *
 * @return  This function returns zero on success, or negative value with error code. \n
 *          Please refer 'mm_error.h' to know it in detail.
 * @pre     None
 * @post    MM_PLAYER_STATE_NULL
 * @see     mm_player_destroy
 * @remark  You can create multiple handles on a context at the same time. \n
 *          However, player cannot guarantee proper operation because of limitation of resources, \n
 *          such as audio device or display device.
 *
 * @par Example
 * @code
char *g_err_attr_name = NULL;

if (mm_player_create(&g_player) != MM_ERROR_NONE) {
	LOGE("failed to create player\n");
}

if (mm_player_set_attribute(g_player,
						&g_err_attr_name,
						"profile_uri", filename, strlen(filename),
						"display_overlay", (void *)&g_win.xid, sizeof(g_win.xid),
						NULL) != MM_ERROR_NONE) {
	LOGE("failed to set %s attribute\n", g_err_attr_name);
	free(g_err_attr_name);
}

mm_player_set_message_callback(g_player, msg_callback, (void *)g_player);
 * @endcode
 */
int mm_player_create(MMHandleType *player);

/**
 * This function releases player object and all resources which were created by mm_player_create(). \n
 * And, player handle will also be destroyed.
 *
 * @param   player [in] Handle of player
 *
 * @return  This function returns zero on success, or negative value with error code.
 * @pre     Player state may be MM_PLAYER_STATE_NULL. \n
 *          But, it can be called in any state.
 * @post    Because handle is released, there is no any state.
 * @see     mm_player_create
 * @remark  This method can be called with a valid player handle from any state to \n
 *          completely shutdown the player operation.
 *
 * @par Example
 * @code
if (mm_player_destroy(g_player) != MM_ERROR_NONE) {
	LOGE("failed to destroy player\n");
}
 * @endcode
 */
int mm_player_destroy(MMHandleType player);

/**
 * This function parses uri and makes gstreamer pipeline by uri scheme. \n
 * So, uri should be set before realizing with mm_player_set_attribute(). \n
 *
 * @param   player [in] Handle of player
 *
 * @return  This function returns zero on success, or negative value with error code.
 *
 * @pre     Player state should be MM_PLAYER_STATE_NULL.
 * @post    player state will be MM_PLAYER_STATE_READY.
 * @see     mm_player_unrealize
 * @remark  None
 * @par Example
 * @code
if (mm_player_realize(g_player) != MM_ERROR_NONE) {
	LOGE("failed to realize player\n");
}
 * @endcode
 */
int mm_player_realize(MMHandleType player);

/**
 * This function uninitializes player object. So, resources and allocated memory \n
 * will be freed. And, gstreamer pipeline is also destroyed. So, if you want to play \n
 * other contents, player should be created again after destruction or realized with new uri.
 *
 * @param   player [in] Handle of player
 *
 * @return  This function returns zero on success, or negative value with error code.
 * @pre     Player state may be MM_PLAYER_STATE_READY to unrealize. \n
 *          But, it can be called in any state.
 * @post    Player state will be MM_PLAYER_STATE_NULL.
 * @see     mm_player_realize
 * @remark  This method can be called with a valid player handle from any state.
 *
 * @par Example
 * @code
if (mm_player_unrealize(g_player) != MM_ERROR_NONE) {
	LOGE("failed to unrealize player\n");
}
 * @endcode
 */
int mm_player_unrealize(MMHandleType player);

/**
 * This function is to abort pause state transition
 * for unrealize or destroy.
 */
int mm_player_abort_pause(MMHandleType player);

/**
 * This function is to get current state of player. \n
 * Application have to check current state before doing some action.
 *
 * @param player [in]  Handle of player
 * @param state  [out] current state of player on success
 *
 * @return   This function returns zero on success, or negative value with error code.
 *
 * @see      mmplayer_state_e
 * @remark   None
 * @par Example
 * @code
if (mm_player_get_state(g_player, &state) != MM_ERROR_NONE) {
	LOGE("failed to get state\n");
}
 * @endcode
 */
int mm_player_get_state(MMHandleType player, mmplayer_state_e *state);

/**
 * This function is to control logical volume.
 */
int mm_player_set_volume(MMHandleType player, float volume);

/**
 * This function is to get current volume factor of player.
 */
int mm_player_get_volume(MMHandleType player, float *volume);

/**
 * This function is to start playing media contents. Demux(parser), codec and related plugins are decided \n
 * at this time. And, MM_MESSAGE_BEGIN_OF_STREAM will be posted through callback function registered \n
 * by mm_player_set_message_callback().
 *
 * @param	player		[in]	Handle of player
 *
 * @return	This function returns zero on success, or negative value with error code.
 * @remark
 *
 * @pre		Player state may be MM_PLAYER_STATE_READY.
 * @post		Player state will be MM_PLAYER_STATE_PLAYING.
 * @see		mm_player_stop
 * @remark  None
 * @par Example
 * @code
if (mm_player_start(g_player) != MM_ERROR_NONE)
{
	LOGE("failed to start player\n");
}
 * @endcode
 */
int mm_player_start(MMHandleType player);

/**
 * This function is to stop playing media contents and it's different with pause. \n
 * If mm_player_start() is called after this, content will be started again from the beginning. \n
 * So, it can be used to close current playback.
 *
 * @param	player		[in]	Handle of player
 *
 * @return	This function returns zero on success, or negative value with error code.
 *
 * @pre		Player state may be MM_PLAYER_STATE_PLAYING.
 * @post		Player state will be MM_PLAYER_STATE_READY.
 * @see		mm_player_start
 * @remark  None
 * @par Example
 * @code
if (mm_player_stop(g_player) != MM_ERROR_NONE)
{
	LOGE("failed to stop player\n");
}
 * @endcode
 */
int mm_player_stop(MMHandleType player);

/**
 * This function is to pause playing media contents.
 *
 * @param	player		[in]	Handle of player.
 *
 * @return	This function returns zero on success, or negative value with error code.
 *
 * @pre		Player state may be MM_PLAYER_STATE_PLAYING.
 * @post		Player state will be MM_PLAYER_STATE_PAUSED.
 * @see		mm_player_resume
 * @remark  None
 * @par Example
 * @code
if (mm_player_pause(g_player) != MM_ERROR_NONE)
{
	LOGE("failed to pause player\n");
}
 * @endcode
 */
int mm_player_pause(MMHandleType player);

/**
 * This function is to resume paused media contents.
 *
 * @param	player		[in]	Handle of player.
 *
 * @return	This function returns zero on success, or negative value with error code.
 *
 * @pre		Player state may be MM_PLAYER_STATE_PAUSED.
 * @post		Player state will be MM_PLAYER_STATE_PLAYING.
 * @see		mm_player_pause
 * @remark  None
 * @par Example
 * @code
if (mm_player_resume(g_player) != MM_ERROR_NONE)
{
	LOGE("failed to resume player\n");
}
 * @endcode
 */
int mm_player_resume(MMHandleType player);

/**
 * This function is to set the position for playback. \n
 * So, it can be seeked to requested position. \n
 *
 * @param	player		[in]	Handle of player
 * @param	pos			[in]	Position for playback
 *
 * @return	This function returns zero on success, or negative value with error code.
 * @see		mm_player_get_position
 * @remark  the unit of time-based format is millisecond and other case is percent.
 */
int mm_player_set_position(MMHandleType player, int64_t pos);

/**
 * This function is to get current position of playback content.
 *
 * @param	player		[in]	Handle of player.
 * @param	format		[in]	Format of position.
 * @param    pos        [out] contains current position on success or zero in case of failure.
 *
 * @return	This function returns zero on success, or negative value with errors
 * @see		mm_player_set_position
 * @remark  the unit of time-based format is millisecond and other case is percent.
 */
int mm_player_get_position(MMHandleType player, int64_t *pos);

/**
 * This function is to get the content time duration.
 */
int mm_player_get_duration(MMHandleType player, int64_t *dur);

/**
 * This function is to get current buffer position of playback content.
 *
 * @param   player      [in]	Handle of player.
 * @param   format      [in]	Format of position.
 * @param   start_pos   [out] contains buffer start  position on success or zero in case of failure.
 * @param   stop_pos    [out] contains buffer current  position on success or zero in case of failure.
 *
 * @return	This function returns zero on success, or negative value with errors
 * @see		mm_player_set_position
 * @remark  the unit of time-based format is millisecond and other case is percent.
 * @par Example
 * @code
int start_pos = 0, stop_pos = 0;

mm_player_get_buffer_position(g_player, &start_pos, &end_pos );

LOGD("buffer position: [%d] ~ [%d] \%\n", start_pos, end_pos );
 * @endcode
 */
int mm_player_get_buffer_position(MMHandleType player, int *start_pos, int *end_pos);

/**
 * This function sets callback function for receiving messages from player.
 * So, player can notify warning, error and normal cases to application.
 *
 * @param	player		[in]	Handle of player.
 * @param	callback	[in]	Message callback function.
 * @param	user_param	[in]	User parameter which is passed to callback function.
 *
 * @return	This function returns zero on success, or negative value with error code.
 * @see		MMMessageCallback
 * @remark  None
 * @par Example
 * @code
int msg_callback(int message, MMMessageParamType *param, void *user_param)
{
	switch (message)
	{
		case MM_MESSAGE_ERROR:
			//do something
			break;

		case MM_MESSAGE_END_OF_STREAM:
			//do something
			break;

		case MM_MESSAGE_STATE_CHANGED:
			//do something
			break;

		case MM_MESSAGE_BEGIN_OF_STREAM:
			//do something
			break;

		default:
			break;
	}
	return TRUE;
}

mm_player_set_message_callback(g_player, msg_callback, (void *)g_player);
 * @endcode
 */
int mm_player_set_message_callback(MMHandleType player, MMMessageCallback callback, void *user_param);

/**
 * This function is to mute volume of player
 *
 * @param	player	[in]	Handle of player
 * @param	mute	[in]	mute value
 *
 * @return	This function returns zero on success, or negative value with error code
 * @see		mm_player_get_mute
 * @remark  None
 */
int mm_player_set_mute(MMHandleType player, bool mute);

/**
 * This function is to get mute value of player
 *
 * @param	player	[in]	Handle of player
 * @param	mute	[out]	Sound is muted
 *
 * @return	This function returns zero on success, or negative value with error code
 * @see		mm_player_set_mute
 * @remark  None
 */
int mm_player_get_mute(MMHandleType player, bool *mute);

/**
 * This function is to adjust subtitle postion. So, subtitle can show at the adjusted position. \n
 * If pos is negative, subtitle will be displayed previous time, the other hand forward time. \n
 *
 * @param	player	[in]	Handle of player
 * @param	pos		[in]	postion to be adjusted
 *
 * @return	This function returns zero on success, or negative value with error
 *			code
 * @see		mm_player_adjust_subtitle_position
 * @remark  None
 * @par Example
 * @code
int pos;

pos = 5000;
if (mm_player_adjust_subtitle_position(g_player, pos) != MM_ERROR_NONE)
{
	LOGW("failed to adjust subtitle postion.\n");
}
 * @endcode
 */

int mm_player_adjust_subtitle_position(MMHandleType player, int pos);

/**
 * This function is to set subtitle silent status. So, subtitle can show or hide during playback \n
 * by this value. But, one subtitle file should be set with "subtitle_uri" attribute before calling mm_player_realize(); \n
 * Player FW parses subtitle file and send text data including timestamp to application \n
 * through message callback with MM_MESSAGE_UPDATE_SUBTITLE will be. \n
 * So, application have to render it. And, subtitle can be supported only in a seprate file. \n
 * So, it's not supported for embedded case.
 *
 * @param	player	[in]	Handle of player
 * @param	silent	[in]	silent(integer value except 0) or not silent(0)
 *
 * @return	This function returns zero on success, or negative value with error
 *			code
 * @see		mm_player_get_subtitle_silent, MM_MESSAGE_UPDATE_SUBTITLE
 * @remark  None
 * @par Example
 * @code
mm_player_set_attribute(g_player,
					&g_err_name,
					"subtitle_uri", g_subtitle_uri, strlen(g_subtitle_uri),
					NULL
					);

if (mm_player_set_subtitle_silent(g_player, TRUE) != MM_ERROR_NONE)
{
	LOGW("failed to set subtitle silent\n");
}
 * @endcode
 */
int mm_player_set_subtitle_silent(MMHandleType player, int silent);

/**
 * This function is to get silent status of subtitle.
 *
 * @param	player	[in]	Handle of player
 * @param	silent	[out]	subtitle silent property
 *
 * @return	This function returns zero on success, or negative value with error
 *			code
 * @see		mm_player_set_subtitle_silent, MM_MESSAGE_UPDATE_SUBTITLE
 * @remark  None
 * @par Example
 * @code
int silent = FALSE;

if (mm_player_get_subtitle_silent(g_player, &silent) != MM_ERROR_NONE)
{
	LOGW("failed to set subtitle silent\n");
}
 * @endcode
 */
int mm_player_get_subtitle_silent(MMHandleType player, int *silent);

/**
 * This function is to set attributes into player. Multiple attributes can be set simultaneously. \n
 * If one of attribute fails, this function will stop at the point and let you know the name which is failed. \n
 *
 * @param   player                 [in]   Handle of player.
 * @param   err_attr_name          [out]  Name of attribute which is failed to set
 * @param   first_attribute_name   [in]   Name of the first attribute to set
 * @param   ...                    [in]   Value for the first attribute, followed optionally by more name/value pairs, terminated by NULL.
 *                                        But, in the case of data or string type, it should be name/value/size.
 *
 * @return  This function returns zero on success, or negative value with error code.
 *
 * @see     mm_player_get_attribute
 * @remark  This function must be terminated by NULL argument.
 *          And, if this function is failed, err_attr_name param must be free.
 * @par Example
 * @code
char *g_err_attr_name = NULL;

if (mm_player_set_attribute(g_player,
						&g_err_attr_name,
						"profile_uri", filename, strlen(filename),
						"profile_play_count", count,
						NULL) != MM_ERROR_NONE) {
	LOGW("failed to set %s attribute\n", g_err_attr_name);
	free(g_err_attr_name);
}

 * @endcode
 */
int mm_player_set_attribute(MMHandleType player,  char **err_attr_name, const char *first_attribute_name, ...)G_GNUC_NULL_TERMINATED;

/**
 * This function is to get attributes from player. Multiple attributes can be got simultaneously.
 *
 * @param   player                [in]  Handle of player.
 * @param   err_attr_name         [out] Name of attribute which is failed to get
 * @param   first_attribute_name  [in]  Name of the first attribute to get
 * @param   ...                   [out] Value for the first attribute, followed optionally by more name/value pairs, terminated by NULL.
 *                                      But, in the case of data or string type, it should be name/value/size.
 *
 * @return  This function returns zero on success, or negative value with error
 *          code.
 * @see     mm_player_set_attribute
 * @remark  This function must be terminated by NULL argument.
 *          And, if this function is failed, err_attr_name param must be free.
 */
int mm_player_get_attribute(MMHandleType player,  char **err_attr_name, const char *first_attribute_name, ...)G_GNUC_NULL_TERMINATED;

/**
 * This function is to get detail information of attribute.
 *
 * @param   player          [in]  Handle of player.
 * @param   attribute_name  [in]  Name of the attribute to get
 * @param   info            [out] Attribute infomation
 *
 * @return  This function returns zero on success, or negative value with error
 *          code.
 *
 * @see     mm_player_set_attribute, mm_player_get_attribute
 * @remark  None
 * @par Example
 * @code
if (mm_player_get_attribute_info(g_player, "display_method", &method_info) != MM_ERROR_NONE) {
	LOGW("failed to get info\n");
}

LOGD("type:%d \n", method_info.type); //int, double..
LOGD("flag:%d \n", method_info.flag); //readable, writable..
LOGD("validity type:%d \n", method_info.validity_type); //range, array..

if (method_info. validity_type == MM_PLAYER_ATTRS_VALID_TYPE_INT_RANGE) {
	LOGD("range min:%d\n", method_info.int_range.min);
	LOGD("range max:%d\n", method_info.int_range.max);
}
 * @endcode
 */
int mm_player_get_attribute_info(MMHandleType player,  const char *attribute_name, mmplayer_attrs_info_t *info);

/**
 * This function is to get the track count
 *
 * @param   player [in]  handle of player.
 * @param   track  [in]  type of the track type
 * @param   info   [out] the count of the track
 *
 * @return  This function returns zero on success, or negative value with error
 *          code.
 *
 * @par Example
 * @code
gint audio_count = 0;

if (mm_player_get_track_count(g_player, MM_PLAYER_TRACK_TYPE_AUDIO, &audio_count) != MM_ERROR_NONE) {
	LOGW("failed to get audio track count\n");
}

LOGD("audio track count : %d \n", audio_count);
 * @endcode
 */
int mm_player_get_track_count(MMHandleType player,  mmplayer_track_type_e type, int *count);

/**
 * This function is to select the track
 *
 * @param   player [in] handle of player.
 * @param   type   [in] type of the track type
 * @param   index  [in] the index of the track
 *
 * @return  This function returns zero on success, or negative value with error
 *          code.
 */
int mm_player_select_track(MMHandleType player, mmplayer_track_type_e type, int index);

/**
 * This function is to get the track language
 *
 * @param   player [in]  handle of player.
 * @param   type   [in]  type of the track type
 * @param   index  [in]  the index of the track
 * @param   code   [out] language code in ISO 639-1(string)
 *
 * @return  This function returns zero on success, or negative value with error
 *          code.
 */
int mm_player_get_track_language_code(MMHandleType player,  mmplayer_track_type_e type, int index, char **code);

/**
 * This function is to get the current running track
 *
 * @param       player          [in]    handle of player.
 * @param       type                    [in]    type of the track type
 * @param       index           [out]    the index of the track
 *
 * @return      This function returns zero on success, or negative value with error
 *                      code.
 */

int mm_player_get_current_track(MMHandleType hplayer, mmplayer_track_type_e type, int *index);

/**
 * This function is to set the subtitle path
 *
 * @param       player  [in]    handle of player
 * @param       path    [in]    subtitle path
 *
 * @return      This function returns zero on success, or negative value with error code.
 */
int mm_player_set_external_subtitle_path(MMHandleType player, const char *path);

/**
 * This function is to set uri.
 *
 * @param   player [in] handle of player
 * @param   uri    [in] uri
 * @return  This function returns zero on success, or negative value with error code.
 */
int mm_player_set_uri(MMHandleType player, const char *uri);

/**
 * This function is to set next uri.
 *
 * @param   player [in] handle of player
 * @param   uri    [in] uri
 * @return  This function returns zero on success, or negative value with error code.
 */
int mm_player_set_next_uri(MMHandleType player, const char *uri);

/**
 * This function is to get next uri.
 *
 * @param   player   [in]  handle of player
 * @param   uri      [out] uri
 * @return  This function returns zero on success, or negative value with error code.
 */
int mm_player_get_next_uri(MMHandleType player, char **uri);

/**
 * This function is to decrease reference count of internal buffer.
 *
 * @param    buffer [in] video callback internal buffer
 * @return   None;
 */
void mm_player_video_stream_internal_buffer_unref(void *buffer);

/**mm_player_submit_packet
 * This function is to submit buffer to appsrc.  \n
 * @param   player			[in]    Handle of player.
 * @param   buf             [in]    buffer to be submit in appsrc in external feeder case.
 * @param   len				[in]	length of buffer.
 * @param   pts				[in]	timestamp of buffer.
 * @param   streamtype		[in]	stream type of buffer.
 * @return  This function returns zero on success, or negative value with error code.
 */
int mm_player_submit_packet(MMHandleType player, media_packet_h packet);

/**mm_player_set_video_info
 * This function is to set caps of src pad of video appsrc in external feeder case.  \n
 * @param   player         [in] Handle of player.
 * @param   media_format_h [in] Video stream info.
 * @return  This function returns zero on success, or negative value with error code.
 */
int mm_player_set_video_info(MMHandleType player, media_format_h format);

/**mm_player_set_audio_info
 * This function is to set caps of src pad of Audio appsrc in external feeder case.  \n
 * @param       player                       [in]    Handle of player.
 * @param       media_format_h               [in]    Audio stream info.
 * @return      This function returns zero on success, or negative value with error code.
 */
int mm_player_set_audio_info(MMHandleType player, media_format_h format);

/**mm_player_set_subtitle_info
 * This function is to set caps of src pad of subtitle appsrc in external feeder case.  \n
 * @param       player                          [in]    Handle of player.
 * @param       subtitle_stream_info               [in]    Subtitle stream info.
 * @return      This function returns zero on success, or negative value with error code.
 */
int mm_player_set_subtitle_info(MMHandleType player, mmplayer_subtitle_stream_info_t *info);

/**
 * This function is to set max size of buffer(appsrc).
 *
 * @param       player          [in]    Handle of player.
 * @param       type            [in]    stream type
 * @param       max_size        [in]    max bytes of buffer.
 *
 * @return      This function returns zero on success, or negative value with error
 *                      code.
 */
int mm_player_set_media_stream_buffer_max_size(MMHandleType player, mmplayer_stream_type_e type, unsigned long long max_size);

/**
 * This function is to get max size of buffer(appsrc).
 *
 * @param       player          [in]    Handle of player.
 * @param       type            [in]    stream type
 * @param       max_size        [out]   max bytes of buffer.
 *
 * @return      This function returns zero on success, or negative value with error
 *                      code.
 */
int mm_player_get_media_stream_buffer_max_size(MMHandleType player, mmplayer_stream_type_e type, unsigned long long *max_size);

/**
 * This function is to set min percent of buffer(appsrc).
 *
 * @param       player          [in]    Handle of player.
 * @param       type            [in]    stream type
 * @param       min_percent     [in]    min percent of buffer.
 *
 * @return      This function returns zero on success, or negative value with error
 *                      code.
 */
int mm_player_set_media_stream_buffer_min_percent(MMHandleType player, mmplayer_stream_type_e type, unsigned min_percent);

/**
 * This function is to get min percent of buffer(appsrc).
 *
 * @param       player          [in]    Handle of player.
 * @param       type            [in]    stream type
 * @param       min_percent     [out]   min percent of buffer.
 *
 * @return      This function returns zero on success, or negative value with error
 *                      code.
 */
int mm_player_get_media_stream_buffer_min_percent(MMHandleType player, mmplayer_stream_type_e type, unsigned int *min_percent);

/**
 * This function set callback function for changing audio stream from player. \n
 * It's only supported when audio stream is included in file. \n
 *
 * @param	player   [in] Handle of player.
 * @param	callback [in] Audio stream changed callback function.
 * @param	user_param [in] User parameter.
 *
 * @return	This function returns zero on success, or negative value with error
 *			code.
 * @see		mm_player_stream_changed_callback
 * @since
 */
int mm_player_set_audio_stream_changed_callback(MMHandleType player, mm_player_stream_changed_callback callback, void *user_param);

/**
 * This function is to get timeout value according to the content type for muse. \n
 * It's only supported when video stream is included in file. \n
 *
 * @param	player  [in] Handle of player.
 * @param	timeout [out] timeout value (sec).
 *
 * @return	This function returns zero on success, or negative value with error
 *			code.
 * @since 3.0
 */
int mm_player_get_timeout(MMHandleType player, int *timeout);

/**
 * This function is to set the dynamic resolution information. \n
 * It's only supported when video stream is included in file. \n
 *
 * @param	player  [in] Handle of player.
 * @param	drc     [in] dynamic resolution info of media stream data
 *
 * @return	This function returns zero on success, or negative value with error
 *			code.
 * @since 3.0
 */
int mm_player_set_media_stream_dynamic_resolution(MMHandleType player, bool drc);

/**
 * This function is to release the video stream bo to reuse. \n
 * It's only supported when sw codec is used to decode video stream. \n
 *
 * @param	player  [in] Handle of player.
 * @param	bo     [in] bo address to be released
 *
 * @return	This function returns zero on success, or negative value with error
 *			code.
 * @since 3.0
 */
int mm_player_release_video_stream_bo(MMHandleType player, void *bo);

/**
 * This function is to set sound stream info
 */
int mm_player_set_sound_stream_info(MMHandleType player, char *stream_type, int stream_index);

/**
 * This function is to manage the playback according to the external storage state
 */
int mm_player_manage_external_storage_state(MMHandleType player, int id, int state);

/**
 * These functions are to set/get the max variant of HAS
 */
int mm_player_get_adaptive_variant_info(MMHandleType player, int *num, char **var_info);
int mm_player_set_max_adaptive_variant_limit(MMHandleType player, int bandwidth, int width, int height);
int mm_player_get_max_adaptive_variant_limit(MMHandleType player, int *bandwidth, int *width, int *height);

/**
 * These functions are to set/get the audio only mode
 */
int mm_player_set_audio_only(MMHandleType player, bool audio_only);
int mm_player_get_audio_only(MMHandleType player, bool *audio_only);

/**
 * These functions are to get the streaming bufferint time
 */
int mm_player_get_streaming_buffering_time(MMHandleType player, int *prebuffer_ms, int *rebuffer_ms);

/**
 * These functions are to display the 360 video content
 */
int mm_player_360_is_content_spherical(MMHandleType player, bool *is_spherical);
int mm_player_360_set_enabled(MMHandleType player, bool enabled);
int mm_player_360_is_enabled(MMHandleType player, bool *enabled);

int mm_player_360_set_direction_of_view(MMHandleType player, float yaw, float pitch);
int mm_player_360_get_direction_of_view(MMHandleType player, float *yaw, float *pitch);

int mm_player_360_set_zoom(MMHandleType player, float level);
int mm_player_360_get_zoom(MMHandleType player, float *level);

int mm_player_360_set_field_of_view(MMHandleType player, int horizontal_degrees, int vertical_degrees);
int mm_player_360_get_field_of_view(MMHandleType player, int *horizontal_degrees, int *vertical_degrees);

/**
 * This function is to set codec type
 */
int mm_player_set_codec_type(MMHandleType player, mmplayer_stream_type_e stream_type, mmplayer_codec_type_e codec_type);

/**
 * These functions are to apply the replaygain
 */
int mm_player_set_replaygain_enabled(MMHandleType player, bool enabled);
int mm_player_is_replaygain_enabled(MMHandleType player, bool *enabled);

/**
 * This function is to set/get video content ROI area
 */
int mm_player_set_video_roi_area(MMHandleType player, double scale_x, double scale_y, double scale_width, double scale_height);
int mm_player_get_video_roi_area(MMHandleType player, double *scale_x, double *scale_y, double *scale_width, double *scale_height);

/**
 * This function is to set play speed for playback.
 *
 * @param	player		[in]	Handle of player.
 * @param	ratio		[in]	Speed for playback.
 * @param	streaming	[in]	If @c true, rate value can be set even if it is streaming playback.
 * @return	This function returns zero on success, or negative value with error
 *			code
 * @remark	The current supported range is from -64x to 64x.
 *		But, the quailty is dependent on codec performance.
 *		And, the sound is muted under normal speed and more than double speed.
 * @see
 * @since
 */
int mm_player_set_play_speed(MMHandleType player, float rate, bool streaming);

/**
 * This function set callback function for receiving video stream from player.
 *
 * @param	player		[in]	Handle of player.
 * @param	callback	[in]	Video stream callback function.
 * @param	user_param	[in]	User parameter.
 *
 * @return	This function returns zero on success, or negative value with error
 *			code.
 * @remark
 * @since
 */
int mm_player_set_video_decoded_callback(MMHandleType player, mm_player_video_decoded_callback callback, void *user_param);

/**
 * This function set callback function for receiving audio stream from player.
 *
 * @param	player		[in]	Handle of player.
 * @param	opt 		[in]	audio extract option.
 * @param	callback	[in]	audio stream callback function.
 * @param	user_param	[in]	User parameter.
 *
 * @return	This function returns zero on success, or negative value with error
 *			code.
 * @remark
 * @see		mm_player_audio_decoded_callback
 * @since
 */
int mm_player_set_audio_decoded_callback(MMHandleType player, mmplayer_audio_extract_opt_e opt, mm_player_audio_decoded_callback callback, void *user_param);

/**
 * This function is to capture video frame.
 *
 * @param	player		[in]	Handle of player.
 *
 * @return	This function returns zero on success, or negative value with error
 *			code.
 *
 * @remark	Captured buffer is sent asynchronously through message callback with MM_MESSAGE_VIDEO_CAPTURED.
 *			And, application should free the captured buffer directly.
 * @see		MM_MESSAGE_VIDEO_CAPTURED
 * @since
 */
int mm_player_do_video_capture(MMHandleType player);

/**
 * This function changes the previous videosink plugin for a new one
 *
 * @param	player			[in]	Handle of player.
 * @param	display_surface_type	[in]	display surface type to set
 * @param	wl_surface_id			[in]	wayland surface id to set
 *
 * @return	This function returns zero on success, or negative value with error
 *			code.
 * @remark
 * @see
 * @since
 */
int mm_player_change_videosink(MMHandleType player, MMDisplaySurfaceType display_surface_type, int wl_surface_id);

/**
 * This function set the application PID.
 *
 * @param	player		[in]	Handle of player
 * @param	pid			[in]	application PID
 *
 * @return	This function returns zero on success, or negative value with error code.
 *
 * @pre		Player state should be MM_PLAYER_STATE_NULL.
 * @remark	None
 */
int mm_player_set_client_pid(MMHandleType player, int pid);

/**
 * This function gets the activation status of audio offload.
 */
int mm_player_audio_offload_is_activated(MMHandleType player, bool *activated);

/**
 * This function checks whether the controlling audio with opt is available or not.
 */
int mm_player_is_audio_control_available(MMHandleType player, mmplayer_audio_control_opt_e opt, bool *available);

/**
	@}
 */

#ifdef __cplusplus
	}
#endif

#endif	/* __MM_PLAYER_H__ */
