/*
 * libmm-player
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: JongHyuk Choi <jhchoi.choi@samsung.com>, heechul jeon <heechul.jeon@samsung.co>,
 * YoungHwan An <younghwan_.an@samsung.com>, Eunhae Choi <eunhae1.choi@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __MM_PLAYER_GST_H__
#define __MM_PLAYER_GST_H__

/*=======================================================================================
| INCLUDE FILES										|
========================================================================================*/
#include <mm_types.h>
#include "mm_player_priv.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* get caps info */
#define MMPLAYER_GST_GET_CAPS_INFO(x_pad, x_caps, x_str, x_name, x_ret) \
	do { \
		if (!x_caps) \
			x_caps = gst_pad_query_caps(x_pad, NULL); \
		if (x_caps) { \
			x_str = gst_caps_get_structure(x_caps, 0); \
			if (x_str) { \
				x_name = gst_structure_get_name(x_str); \
			} \
		} \
		if (!x_caps || !x_str || !x_name) { \
			LOGE("failed to get caps info"); \
			x_ret = FALSE; \
		} \
	} while (0)

/*=======================================================================================
| GLOBAL FUNCTION PROTOTYPES								|
========================================================================================*/

int _mmplayer_gst_set_state(mmplayer_t *player, GstElement *element,  GstState state, gboolean async, gint timeout);

int _mmplayer_gst_start(mmplayer_t *player);
int _mmplayer_gst_stop(mmplayer_t *player);
int _mmplayer_gst_pause(mmplayer_t *player, gboolean async);
int _mmplayer_gst_resume(mmplayer_t *player, gboolean async);
gboolean _mmplayer_gst_send_event_to_sink(mmplayer_t *player, GstEvent *event);
gboolean _mmplayer_gst_seek(mmplayer_t *player, GstElement *element, gdouble rate,
			GstFormat format, GstSeekFlags flags, GstSeekType cur_type,
			gint64 cur, GstSeekType stop_type, gint64 stop);
int _mmplayer_gst_set_position(mmplayer_t *player, gint64 position, gboolean internal_called);
int _mmplayer_gst_get_position(mmplayer_t *player, gint64 *position);
int _mmplayer_gst_get_buffer_position(mmplayer_t *player, int *start_pos, int *end_pos);
GstElement *_mmplayer_gst_create_source(mmplayer_t *player);
int _mmplayer_gst_build_es_pipeline(mmplayer_t *player);
int _mmplayer_gst_build_pipeline(mmplayer_t *player);
int _mmplayer_gst_add_bus_watch(mmplayer_t *player);
void _mmplayer_activate_next_source(mmplayer_t *player, GstState target);

#ifdef __cplusplus
}
#endif

#endif /* __MM_PLAYER_GST_H__ */
