/*
 * libmm-player
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: JongHyuk Choi <jhchoi.choi@samsung.com>, YeJin Cho <cho.yejin@samsung.com>,
 * Seungbae Shin <seungbae.shin@samsung.com>, YoungHwan An <younghwan_.an@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __MM_PLAYER_INI_H__
#define __MM_PLAYER_INI_H__

#include <glib.h>
#include <mm_types.h>
#include "mm_player_audioeffect.h"

#ifdef __cplusplus
	extern "C" {
#endif

#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "MM_PLAYER"

#define MM_PLAYER_INI_DEFAULT_PATH	SYSCONFDIR"/multimedia/mmfw_player.ini" /* SYSCONFDIR is defined at .spec */
#define MM_PLAYER_INI_DEFAULT_AUDIOEFFECT_PATH	SYSCONFDIR"/multimedia/mmfw_player_audio_effect.ini" /* SYSCONFDIR is defined at .spec */

#define PLAYER_INI_MAX_STRLEN       100
#define PLAYER_INI_MAX_PARAM_STRLEN 256

#define PLAYER_INI_MAX_ELEMENT      10
#define PLAYER_INI_MAX_DEVICE_TYPE  10
#define PLAYER_INI_MAX_SUPPORTED_MEDIA_FORMAT 10 /* @see MAX_SUPPORTED_MEDIA_FORMAT in player_private.h */

/* NOTE : MMPlayer has no initalizing API for library itself
 * so we cannot decide when those ini values to be released.
 * this is the reason of all string items are static array.
 * make it do with malloc when MMPlayerInitialize() API created
 * before that time, we should be careful with size limitation
 * of each string item.
 */
enum keyword_type {
	KEYWORD_EXCLUDE,    // for element exclude keyword
	KEYWORD_DUMP,       // for dump element keyword
	KEYWORD_UNSUPPORTED_CODEC, // for un-supported codec
	KEYWORD_A_SW_CODEC, // for audio sw codec
	KEYWORD_V_SW_CODEC, // for video sw codec
	KEYWORD_A_OFFLOAD_DEVICE_TYPE,  // for audio device type which can support offload
	KEYWORD_A_OFFLOAD_MEDIA_FORMAT, // supported media format for audio offload
	KEYWORD_MEDIA_STREAM_IN_FORMAT, // supported media format for external media stream
};

typedef struct {
	/* general */
	gchar videosink_element_overlay[PLAYER_INI_MAX_STRLEN];
	gchar videosink_element_fake[PLAYER_INI_MAX_STRLEN];
	gchar videoconverter_element[PLAYER_INI_MAX_STRLEN];
	gchar videocodec_element_hw[PLAYER_INI_MAX_STRLEN];
	gchar videocodec_element_sw[PLAYER_INI_MAX_ELEMENT][PLAYER_INI_MAX_STRLEN];
	gchar videocodec_default_type[PLAYER_INI_MAX_STRLEN];
	gchar audiocodec_element_hw[PLAYER_INI_MAX_STRLEN];
	gchar audiocodec_element_sw[PLAYER_INI_MAX_ELEMENT][PLAYER_INI_MAX_STRLEN];
	gchar audiocodec_default_type[PLAYER_INI_MAX_STRLEN];
	gchar audioresampler_element[PLAYER_INI_MAX_STRLEN];
	gchar audiosink_element[PLAYER_INI_MAX_STRLEN];
	gchar audio_offload_sink_element[PLAYER_INI_MAX_STRLEN];
	gchar audio_offload_device_type[PLAYER_INI_MAX_DEVICE_TYPE][PLAYER_INI_MAX_STRLEN];
	gchar audio_offload_media_format[PLAYER_INI_MAX_SUPPORTED_MEDIA_FORMAT][PLAYER_INI_MAX_STRLEN];
	gchar media_stream_input_format[PLAYER_INI_MAX_SUPPORTED_MEDIA_FORMAT][PLAYER_INI_MAX_STRLEN];
	gboolean skip_rescan;
	gboolean generate_dot;
	gboolean use_system_clock;
	gint live_state_change_timeout;
	gint localplayback_state_change_timeout;
	gint delay_before_repeat;
	gint eos_delay;

	gchar gst_param[5][PLAYER_INI_MAX_PARAM_STRLEN];
	gchar exclude_element_keyword[PLAYER_INI_MAX_ELEMENT][PLAYER_INI_MAX_STRLEN];
	gchar unsupported_codec_keyword[PLAYER_INI_MAX_ELEMENT][PLAYER_INI_MAX_STRLEN];
	gboolean disable_segtrap;
	gint pcm_buffer_size;
	gint num_of_video_bo;
	gint video_bo_timeout;
	gboolean support_audio_effect;
	gboolean support_pcm_exporting;
	gboolean support_pitch_control;
	gboolean support_replaygain_control;

	/* http streaming */
	gchar httpsrc_element[PLAYER_INI_MAX_STRLEN];
	guint http_ring_buffer_size;
	gint http_timeout;

	/* audio effect */
	gchar audioeffect_element[PLAYER_INI_MAX_STRLEN];
	gchar audioeffect_element_custom[PLAYER_INI_MAX_STRLEN];

	/* audio effect preset mode */
	gboolean use_audio_effect_preset;
	gboolean audio_effect_preset_list[MM_AUDIO_EFFECT_PRESET_NUM];
	gboolean audio_effect_preset_earphone_only_list[MM_AUDIO_EFFECT_PRESET_NUM];

	/* audio effect custom mode */
	gboolean use_audio_effect_custom;
	gboolean audio_effect_custom_list[MM_AUDIO_EFFECT_CUSTOM_NUM];
	gboolean audio_effect_custom_earphone_only_list[MM_AUDIO_EFFECT_CUSTOM_NUM];
	gint audio_effect_custom_eq_band_num;
	gint audio_effect_custom_eq_band_width[MM_AUDIO_EFFECT_EQ_BAND_NUM_MAX];
	gint audio_effect_custom_eq_band_freq[MM_AUDIO_EFFECT_EQ_BAND_NUM_MAX];
	gint audio_effect_custom_ext_num;
	gint audio_effect_custom_min_level_list[MM_AUDIO_EFFECT_CUSTOM_NUM];
	gint audio_effect_custom_max_level_list[MM_AUDIO_EFFECT_CUSTOM_NUM];

	/* dump buffer for debug */
	gchar dump_element_keyword[PLAYER_INI_MAX_ELEMENT][PLAYER_INI_MAX_STRLEN];
	gchar dump_element_path[PLAYER_INI_MAX_STRLEN];
	gboolean set_dump_element_flag;
} mmplayer_ini_t;

/* default values if each values are not specified in inifile */
/* general */
#define DEFAULT_EMPTY_VALUE                               ""
#define DEFAULT_AUDIO_EFFECT_ELEMENT                      ""
#define DEFAULT_USE_AUDIO_EFFECT_PRESET                   FALSE
#define DEFAULT_AUDIO_EFFECT_PRESET_LIST                  ""
#define DEFAULT_AUDIO_EFFECT_PRESET_LIST_EARPHONE_ONLY    ""
#define DEFAULT_USE_AUDIO_EFFECT_CUSTOM                   FALSE
#define DEFAULT_AUDIO_EFFECT_CUSTOM_LIST                  ""
#define DEFAULT_AUDIO_EFFECT_CUSTOM_LIST_EARPHONE_ONLY    ""
#define DEFAULT_AUDIO_EFFECT_CUSTOM_EQ_BAND_NUM           0
#define DEFAULT_AUDIO_EFFECT_CUSTOM_EQ_BAND_WIDTH         ""
#define DEFAULT_AUDIO_EFFECT_CUSTOM_EQ_BAND_FREQ          ""
#define DEFAULT_AUDIO_EFFECT_CUSTOM_EQ_MIN                0
#define DEFAULT_AUDIO_EFFECT_CUSTOM_EQ_MAX                0
#define DEFAULT_AUDIO_EFFECT_CUSTOM_EXT_NUM               0
#define DEFAULT_USE_SINK_HANDLER                          TRUE
#define DEFAULT_SKIP_RESCAN                               TRUE
#define DEFAULT_GENERATE_DOT                              FALSE
#define DEFAULT_USE_SYSTEM_CLOCK                          TRUE
#define DEFAULT_DELAY_BEFORE_REPEAT                       50 /* msec */
#define DEFAULT_EOS_DELAY                                 0 /* msec */
#define DEFAULT_VIDEOSINK_OVERLAY                         "tizenwlsink"
#define DEFAULT_VIDEOSINK_EVAS                            "evasimagesink"
#define DEFAULT_VIDEOSINK_FAKE                            "fakesink"
#define DEFAULT_AUDIORESAMPLER                            "audioresample"
#define DEFAULT_AUDIOSINK                                 "pulsesink"
#define DEFAULT_A_OFFLOAD_DEVICE_TYPE                     ""
#define DEFAULT_GST_PARAM                                 ""
#define DEFAULT_EXCLUDE_KEYWORD                           ""
#define DEFAULT_UNSUPPORTED_CODEC_KEYWORD                 ""
#define DEFAULT_DISABLE_SEGTRAP                           TRUE
#define DEFAULT_VIDEO_CONVERTER                           ""
#define DEFAULT_LIVE_STATE_CHANGE_TIMEOUT                 30 /* sec */
#define DEFAULT_LOCALPLAYBACK_STATE_CHANGE_TIMEOUT        10 /* sec */
#define DEFAULT_PCM_BUFFER_SIZE                           51200 /* bytes */
#define DEFAULT_NUM_OF_VIDEO_BO                           10
#define DEFAULT_TIMEOUT_OF_VIDEO_BO                       10 /* sec */
#define DEFAULT_AUDIO_OFFLOAD_SINK                        ""
#define DEFAULT_SUPPORT_AUDIO_EFFECT                      FALSE /* with HW audio codec */
#define DEFAULT_SUPPORT_PCM_EXPORTING                     FALSE /* with HW audio codec */
#define DEFAULT_SUPPORT_PITCH_CONTROL                     FALSE /* with HW audio codec */
#define DEFAULT_SUPPORT_REPLAYGAIN_CONTROL                FALSE /* with HW audio codec */

/* http streaming */
#define DEFAULT_HTTPSRC                                   "souphttpsrc"
#define DEFAULT_HTTP_RING_BUFFER_SIZE                     (20*1024*1024) /* bytes : 20MBytes */
#define DEFAULT_HTTP_TIMEOUT                              -1        /* infinite retry */

/* dump buffer for debug */
#define DEFAULT_DUMP_ELEMENT_KEYWORD                      ""
#define DEFAULT_DUMP_ELEMENT_PATH                         "/tmp/"

int
mm_player_ini_load(mmplayer_ini_t *ini);

int
mm_player_audio_effect_ini_load(mmplayer_ini_t *ini);

#ifdef __cplusplus
	}
#endif

#endif
