/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include <iostream>
#include "mm_error.h"
#include "mm_player_priv.h"
#include "gtest_mm_player.h"
#include "gtest_mm_player_priv.h"

using ::testing::InitGoogleTest;
using ::testing::Test;
using ::testing::TestCase;

TEST(MMPlayerPreTest, Create_p)
{
	int ret = MM_ERROR_NONE;
	MMPlayer player;

	ret = player.Create();
	EXPECT_EQ(ret, MM_ERROR_NONE);
}

TEST(MMPlayerPreTest, Destroy_p)
{
	int ret = MM_ERROR_NONE;
	MMPlayer player;

	ret = player.Create();
	EXPECT_EQ(ret, MM_ERROR_NONE);

	ret = player.Destroy();
	EXPECT_EQ(ret, MM_ERROR_NONE);
}

TEST(MMPlayerPrePrivTest, Create_p)
{
	int ret = MM_ERROR_NONE;
	MMPlayerPriv player;

	ret = player.Create();
	EXPECT_EQ(ret, MM_ERROR_NONE);
}

TEST(MMPlayerPrePrivTest, Destroy_p)
{
	int ret = MM_ERROR_NONE;
	MMPlayerPriv player;

	ret = player.Create();
	EXPECT_EQ(ret, MM_ERROR_NONE);

	ret = player.Destroy();
	EXPECT_EQ(ret, MM_ERROR_NONE);
}

class MMPlayerPrivTest : public ::testing::Test {
	public:
		virtual void SetUp();
		virtual void TearDown();

	protected:
		MMPlayerPriv priv_player;
};

void MMPlayerPrivTest::SetUp()
{
	int ret = MM_ERROR_NONE;
	ret = priv_player.Create();
	EXPECT_EQ(ret, MM_ERROR_NONE);
}

void MMPlayerPrivTest::TearDown()
{
	int ret = MM_ERROR_NONE;
	mmplayer_state_e state = MM_PLAYER_STATE_NULL;

	ret = priv_player.GetCurrentState(&state);
	if (ret != MM_ERROR_NONE)
		LOGE("failed to get current state");

	if (state > MM_PLAYER_STATE_NULL) {
		ret = priv_player.Unrealize();
		if (ret != MM_ERROR_NONE)
			LOGE("failed to unrealize");
	}

	ret = priv_player.Destroy();
	if (ret != MM_ERROR_NONE)
		LOGE("failed to destroy");
}

TEST_F(MMPlayerPrivTest, Realize_P)
{
	int ret = MM_ERROR_NONE;
	const gchar *uri = "http://test-url.mp4";

	ret = priv_player.SetUri(uri);
	EXPECT_EQ(ret, MM_ERROR_NONE);

	ret = priv_player.Realize();
	EXPECT_EQ(ret, MM_ERROR_NONE);
}

TEST_F(MMPlayerPrivTest, Realize_N)
{
	int ret = MM_ERROR_NONE;
	ret = priv_player.Realize();
	EXPECT_EQ(ret, MM_ERROR_PLAYER_INVALID_URI);
}

TEST_F(MMPlayerPrivTest, ParseProfile_p)
{
	int ret = MM_ERROR_NONE;
	const gchar *uri = "http://test-url";
	int uri_type = MM_PLAYER_URI_TYPE_NONE;

	ret = priv_player.ParseProfile(uri, &uri_type);

	EXPECT_EQ(ret, MM_ERROR_NONE);
	EXPECT_EQ(uri_type, MM_PLAYER_URI_TYPE_URL_HTTP);
}

TEST_F(MMPlayerPrivTest, ParseProfile_n)
{
	int ret = MM_ERROR_NONE;
	const gchar *uri = "file://non-existing-file.mp4";

	ret = priv_player.ParseProfile(uri, NULL);

	EXPECT_EQ(ret, MM_ERROR_PLAYER_FILE_NOT_FOUND);
}

gint main(gint argc, gchar **argv)
{
	InitGoogleTest(&argc, argv);

	return RUN_ALL_TESTS();
}
